######################################################################################################################
# Preference Reversal Study Data Processing File                                          		   Collins and James #
######################################################################################################################
#																													 #
# 	Sean M. Collins							Duncan James															 #
# 	Assistant Professor of Economics		Associate Professor of Economics										 #
# 	Fordham University						Fordham University														 #
#																													 #
# 	113 W 60th St Rm 924					441 East Fordham Road													 #
# 	New York NY 10023-7414					Bronx, NY 10458															 #
# 	(646) 312-8270							(718) 817-4053															 #
# 	scollins15@fordham.edu					dujames@fordham.edu														 #
#																													 #
######################################################################################################################
#																													 #
# This program requires Oliver Kirchkamp's z-tree function. (We used version 2009-12-14.)							 #
# We do not have permission to redistribute it, but it can be found at: http://www.kirchkamp.de/lab/zTree.R  		 #
#																													 #
######################################################################################################################

# Set working directory and remove all objects. (Remember to set your working directory!)
# setwd("");
# rm(list=ls(all=TRUE))

# Load data into R using Oliver Kirchkamp's z-tree function.
source("zTree.R");
x <- zTreeTables ( c("120209_1601.xls","120215_1105.xls","120216_1507.xls","120229_1454.xls","120301_1558.xls","120307_1501.xls","120308_1241.xls","120308_1605.xls","120312_0835.xls","120312_1449.xls","120313_0824.xls","120314_0851.xls","120612_1659.xls"), c("globals","subjects","contracts","session"));

# Subset the tables to the last period data set (with complete data matrix).
x.globals  <- subset(x$globals,x$globals$Period==max(x$globals$Period))																	# Contains global variables such as as parameters.
x.subjects <- subset(x$subjects,x$subjects$Period==max(x$subjects$Period))																# Contains subject choice variables.
# x.contracts <- subset(x$contracts,x$contracts$Period==max(x$contracts$Period))														# Contains subject input decision times. (This should not be converted to last period).

# Create variables from globals that will be useful.
x.session    <- match(match(x.subjects$Date,unique(x.subjects$Date)),match(x.globals$Date,unique(x.globals$Date)))  					# Vector of indices for conversion of global to subject variable.
x.parameters  <- c("HoutcomeA","HprobA","Hupfront","HrefprobA")																			# List of parameters in main data matrix.
x.bdm         <- c("HCE","HPE","HPL","HPR")																								# List of BDM treatments in main data matrix.
x.binary      <- c("BCE","BPE","BPL","BPR")																								# List of binary choice corresponding to BDM treatments in main data matrix.
x.checks      <- c("RCE","RPE","RPL","RPR")																								# List of BDM treatments in time consistency data matrix.
x.rH <- c(2,4,7,9,12,14,17,19)																											# Corresponding main data index for index of time consistency data.

# Create x.output data frame.
x.output <- data.frame(row.names=as.character(1:dim(x.subjects)[1]));																	# Create a data frame with rows equal to the number of subjects.
x.output$subject <- as.integer(rownames(x.output));																						# Create a column for subject number; this is equivalent to row number.
x.output$session <- match(x.subjects$Date,unique(x.subjects$Date));																		# Create a column for session number, first through last.
x.output$order <- as.character((x.globals$"order[1]"+10*x.globals$"order[2]"+100*x.globals$"order[3]")[x.session])						# Create a variable indicating treatment order.
for (column in x.parameters) 																											# Names of globals main parameter columns to be copied.
{
	for (i in 1:22)
	{
		x.output[,dim(x.output)[2]+1] <- (x.globals[,match(paste(column,sprintf("[%i]", i),sep=""),colnames(x.globals))])[x.session];	# Copy column with desired name and index into right-most column of x.output.
		colnames(x.output)[dim(x.output)[2]] <- paste(column,sprintf("[%i]", i),sep="");												# Set name of new column in x.output.
		x.output[which(x.output[,dim(x.output)[2]]==-1),dim(x.output)[2]] <- NA															# Turn "-1"'s into NAs to prevent errors later on. (This will generate warnings, which may be ignored.)
	}
}
for (column in x.bdm) 																													# Names of subjects main data columns to be copied.
{
	for (i in 1:22)
	{
		x.output[,dim(x.output)[2]+1] <- (x.subjects[,match(paste(column,sprintf("[%i]", i),sep=""),colnames(x.subjects))]);			# Copy column with desired name and index into right-most column of x.output.
		colnames(x.output)[dim(x.output)[2]] <- paste(column,sprintf("[%i]", i),sep="");												# Set name of new column in x.output.
		x.output[which(x.output[,dim(x.output)[2]]==-1),dim(x.output)[2]] <- NA															# Turn "-1"'s into NAs to prevent errors later on. (This will generate warnings, which may be ignored.)
	}
}
for (column in x.binary) 																												# Names of subjects main data columns to be copied.
{
	for (i in 1:22)
	{
		x.output[,dim(x.output)[2]+1] <- (x.subjects[,match(paste(column,sprintf("[%i]", i),sep=""),colnames(x.subjects))]);			# Copy column with desired name and index into right-most column of x.output.
		colnames(x.output)[dim(x.output)[2]] <- paste(column,sprintf("[%i]", i),sep="");												# Set name of new column in x.output.
		x.output[which(x.output[,dim(x.output)[2]]==-1),dim(x.output)[2]] <- NA															# Turn "-1"'s into NAs to prevent errors later on. (This will generate warnings, which may be ignored.)
	}
}
# for (column in c("RoutcomeA","RprobA","Rupfront","RrefprobA"))																		# Names of globals time consistency columns to be copied.
# {
# 	for (i in 1:8)
# 	{
# 		x.output[,dim(x.output)[2]+1] <- (x.globals[,match(paste(column,sprintf("[%i]", i),sep=""),colnames(x.globals))])[x.session];	# Copy column with desired name and index into right-most column of x.output.
# 		colnames(x.output)[dim(x.output)[2]] <- paste(column,sprintf("[%i]", x.rH[i]),sep="");											# Set name of new column in x.output.
# 		x.output[which(x.output[,dim(x.output)[2]]==-1),dim(x.output)[2]] <- NA															# Turn "-1"'s into NAs to prevent errors later on. (This will generate warnings, which may be ignored.)
# 	}
# }
for (column in x.checks) 																												# Names of subjects time consistency columns to be copied.
{
	for (i in 1:8)
	{
		x.output[,dim(x.output)[2]+1] <- (x.subjects[,match(paste(column,sprintf("[%i]", i),sep=""),colnames(x.subjects))]);			# Copy column with desired name and index into right-most column of x.output.
		colnames(x.output)[dim(x.output)[2]] <- paste(column,sprintf("[%i]", x.rH[i]),sep="");											# Set name of new column in x.output.
		x.output[which(x.output[,dim(x.output)[2]]==-1),dim(x.output)[2]] <- NA															# Turn "-1"'s into NAs to prevent errors later on. (This will generate warnings, which may be ignored.)
	}
}

# Export data as csv file, omitting NAs.
write.csv(x.output[,complete.cases(t(x.output))], file="data.csv", row.names=FALSE);

# Export csv files for each subject containing data matrix.
for (i in 1:60)
{
	x.Houtput <- matrix(as.numeric(x.output[i,match("HoutcomeA[1]",colnames(x.output)):match("BPR[22]",colnames(x.output))]),22)								# Create a main data matrix for subject i from the row in the data.
	x.Xoutput <- matrix(as.numeric(x.output[i,match(sprintf("RCE[%i]", x.rH[1]),colnames(x.output)):match(sprintf("RPR[%i]", x.rH[8]),colnames(x.output))]),8)	# Create a time consistency data matrix for subject i from the row in the data.

	x.Routput <- data.frame(matrix(NA,22,4))																													# Create an empty matrix for time consistency data.
	for (j in 1:8){ x.Routput[x.rH[j],] <- data.frame(t(x.Xoutput[j,])) }																						# Write time consistency data in rows that conform to main data matrix.
	# for (j in 1:8){ x.Routput[x.rH[j],] <- data.frame(matrix(x.output[i,match(sprintf("RCE[%i]", x.rH[1]),colnames(x.output)):match(sprintf("RPR[%i]", x.rH[8]),colnames(x.output))],8)[j,]) }

	x.subject <- data.frame(cbind(x.Houtput,x.Routput), row.names=1:22)																							# Bind the main and time consistency data matrix together.
	colnames(x.subject) <- c(x.parameters,x.bdm,x.binary,x.checks);																								# Create column names for parameters and data.

	write.csv(x.subject, file=sprintf("subject_%i.csv", i),row.names=FALSE);																					# Write each subject's data separately.
}
write.csv(x.output[match("subject",colnames(x.output)):match("order",colnames(x.output))], file="subject_list.csv", row.names=FALSE);							# Write a list of subject number, session, and treatment for reference.